use std::env;
use std::fs::File;
use std::io::{Read, Write};

extern crate tenquestionmarks;
use tenquestionmarks::Tenquestionmarks;

extern crate toml;

#[macro_use]
extern crate log;
extern crate env_logger;

extern crate time;

fn init_logger () {
    let mut builder = env_logger::Builder::new();
    builder.format(|buf, record: &log::Record| {
        let t = time::now();
        writeln!(buf, "{} {}:{}: {}", time::strftime("%Y-%m-%d %H:%M:%S", &t).unwrap(), record.level(), record.module_path().unwrap_or("?"), record.args())
    }).filter(None, log::LevelFilter::Info);;

    if env::var("RUST_LOG").is_ok() {
       builder.parse(&env::var("RUST_LOG").unwrap());
    }

    builder.init();
}

fn main () {
    init_logger();

    let config_file_name = env::args().nth(1).unwrap_or("tenquestionmarks.toml".into());
    if let Ok(mut file) = File::open(&config_file_name) {
        let mut contents = String::new();
        if let Err(e) = file.read_to_string(&mut contents) {
            error!("Failed to open config file {}: {:?}", config_file_name, e);
        } else {
            match toml::from_str(&contents) {
                Ok(configuration) => {
                    info!("Loaded configuration from: {}", config_file_name);
                    match Tenquestionmarks::from_configuration(configuration) {
                        Ok(tqm) => {
                            info!("tenquestionmarks initialized successfully");
                            tqm.run();
                        },
                        Err(e) => error!("Failed to initialize tenquestionmarks: {:?}", e)
                    }
                },
                Err(error) => error!("Failed to parse config file {}: {:?}. Config file must be a valid TOML file.", config_file_name, error)
            }
        }
    } else {
        error!("Failed to open config file! Please specify path to a config file.");
    }
}
