use modules::EventLoop;
use toml::value::Table;

use std::sync::Arc;
use std::sync::mpsc::Receiver;
use transformable_channels::mpsc::ExtSender;

use helpers::command::split_command;
use event::{Event, Envelope};
use echobox::Echobox;

pub struct EchoboxModule {
    prefix: String,
    file: String
}

impl EchoboxModule {
    pub fn new (_: &Table, configuration: &Table) -> Box<EventLoop> {
        let prefix = configuration.get("prefix")
                                  .and_then(|value| value.as_str())
                                  .unwrap_or("?echobox");

        let file = configuration.get("responses")
                                .and_then(|value| value.as_str())
                                .unwrap_or("echobox.db");

        Box::new(EchoboxModule {
            prefix: String::from(prefix),
            file: String::from(file)
        })
    }
}

impl EventLoop for EchoboxModule {
    fn run(&self, _: Box<ExtSender<Event>>, receiver: Receiver<Arc<Envelope>>) {
        let echobox = Echobox::with_file(&self.file).unwrap();

        loop {
            match receiver.recv() {
                Ok(envelope) => {
                    match envelope.event {
                        Event::Message { ref message } => {
                            debug!("Received message from module {:?}... {:?}", envelope.from, message.content);
                            match split_command(&message.content) {
                                Some((command, in_quote)) => {
                                    if command == self.prefix {
                                        if !in_quote.is_empty() {
                                            match echobox.echo(in_quote) {
                                                Ok(out_quote) => message.reply(&format!("**Echobox, quote #{}:** {}", out_quote.id, out_quote.content)),
                                                Err(error) => { error!("Error from echobox.echo(): {:?}", error) }
                                            }
                                        } else {
                                            message.reply("**Echobox:** Please enter a quote.");
                                        }
                                    }
                                },
                                _ => {}
                            }
                        }
                        _ => ()
                    }
                }
                Err(error) => { error!("Error {:?}", error) }
            }
        }
    }
}
